

#    experiment run control for CAEN - taken from sys.tcl

global MIDAS_DataAcquisitionControlServer_Verbose
set MIDAS_DataAcquisitionControlServer_Verbose 0


variable SACPres 0
variable V1495Pres 0

variable Options

variable SpecBase [file join $env(MIDASBASE) SpecData CAEN]
variable SpecPath

variable Supported_Modules [list V1720 V1724 V1724DPP DT5780 V1730 V1740 V1742 V1751 V1751DPP]
#variable Supported_Modules [list V1740 V1751]

proc load_Options {} {LoadOptions}
proc update_Options {} {SaveOptions}


   proc Set_Options {} {
      variable Options
      variable Supported_Modules

      LoadOptions

      if {![info exists Options(Rate.channels)]} {set Options(Rate.channels) 150}
      if {![info exists Options(Stat.channels)]} {set Options(Stat.channels) 150}
      if {![info exists Options(Stat.shift)]} {set Options(Stat.shift) 6}
      if {![info exists Options(Stat.offset)]} {set Options(Stat.offset) 64}

   if {![info exists Options(VME.settings)]} {set Options(VME.settings) undefined}

      foreach module $Supported_Modules {
         set Options(Include.$module)  1
         set Options(Histogram.$module) 1
      }

      SaveOptions
   }

   proc Dump_Raw_Data {} {
      variable Supported_Modules

#      foreach module $Supported_Modules {append info [Dump_Raw_[set module]_Data]}

      set maxmodule 0
      foreach module $Supported_Modules {
         for {set i 1} {$i <= 16} {incr i} {
            set z [catch {ReadVMERegisterAttr [string tolower $module]#$i.VMEStatus} m]
            if {$z!=0} then {break}
            incr maxmodule
         }
      }

    set info "Raw data dump for CAEN V17XX modules\n"

    for {set i 1} {$i <= $maxmodule} {incr i} {

       append info "V17XX#$i\n"

        DataAcqCommand 19 "Getting Raw Data" "1 $i" 2
        set data [ReadNetVar NetVar.EXEC.Response]

        foreach word $data {
           if {[string range $word 0 1] != "0x"} then {set word 0x$word}
           if {$word == 0xaa5555aa} then {
               append info "$word - end of dump\n"
               break
           }
           if {$word == 0x55aaaa55} then {
               append info "$word - end of event\n"
               continue
           }
           append info "$word "
        }

        append info "\n"
     }

     return $info

   }

   proc Stop_Electronics {} {
       variable Options 
       variable SACPres 
       variable V1495Pres
       variable Supported_Modules

       if {$SACPres != 0} {WriteVMERegister sac#1.STR 0}

       if {$Options(Usepatterngen) == 1} {
          set z [catch {WriteVMERegister patterngen#1.Start  0} m]
          if {$z == 0} {
             puts "waiting after setting patterngen => 0"
              after 1000    ;#  needed because of V1724 firmware problems
          }
       }

       if {$V1495Pres != 0} {
          WriteVMERegister v1495#1.Config.B8 0    ;#  disable acquisiition
          puts "waiting after setting v1495 => 0"
          after 1000   ;#  needed because of V1724 firmware problems 
       }

       foreach m $Supported_Modules {[set m]_DAQ-halt}

     after 2000   ;#  allow DA program to flush data 

   }

   proc Go_Electronics {} {
      variable Options 
      variable SACPres 
      variable V1495Pres
      variable Supported_Modules

       if {$SACPres != 0} {           ;#  SAC installed 

           WriteVMERegister sac#1.Mode 1   ;#   CAEN mode
           WriteVMERegister sac#1.MON0 2
           WriteVMERegister sac#1.STR 9
           WriteVMERegister sac#1.ARM 0
       }

puts "Usepatterngen=$Options(Usepatterngen); Use1495=$Options(Use1495)"

       if {$Options(Usepatterngen) == 1} {
           set z [catch {WriteVMERegister patterngen#1.Start  3} m]
       }

       WriteNetVar NetVar.V1495_UseCtrl  $Options(Use1495)

       if {$V1495Pres != 0} {
          WriteVMERegister v1495#1.TimeStampReset  1

          WriteVMERegister v1495#1.Config.B9  $Options(Use1495)   ;#  set single shot mode

          WriteVMERegister v1495#1.Config.B8  1   ;#  enable acquisition

          WriteVMERegister v1495#1.Arm  $Options(Use1495)         ;#  arm
       }

       foreach m $Supported_Modules {[set m]_DAQ-go}
   }
 
   proc Reset_Electronics {} {    ;#   used by RESET
      variable SACPres 
      variable V1495Pres
      variable Supported_Modules
 
      if {$SACPres != 0} {WriteVMERegister sac#1.EVCLR 0}
   
      if {$V1495Pres != 0} {WriteVMERegister v1495#1.Config.B8 0}  ;# disable acquisition

      foreach m $Supported_Modules {[set m]_DAQ-reset}
   }

   proc Setup_Electronics {} {    ;#   used by SETUP
       variable Supported_Modules

       define_VME_addresses   ;#  setup VME 

       foreach m $Supported_Modules {[set m]_DAQ-setup}

   }

   proc define_VME_addresses {} {
       variable Supported_Modules
       variable Options

#    load hardware configuration into readout software

       ClearConfig
 
       foreach module $Supported_Modules {
          if {$Options(Include.$module) == 1} then {define_[set module]_config}
       }

#    then the acquisition control module

       define_sac_config
       define_v1495_config

       DataAcqCommand 4 "print config" 0 2

   }

   proc define_sac_config {} {
      variable SACPres

      set z [catch {ReadVMERegisterAttr sac#1.STR} m]
      if {$z == 0} then {set SACPres 1} else {set SACPres 0}

      if {$SACPres != 0} {DefineConfig "0 0 [string range [first $m] 2 end]"}

   }

   proc define_v1495_config {} {
      variable V1495Pres

      set z [catch {ReadVMERegisterAttr v1495#1.Ctrl} m]
      if {$z==0} then {set V1495Pres 1} else {set V1495Pres 0}

      if {$V1495Pres != 0} {DefineConfig "1495 0 [string range [first $m] 2 end]"}

   }


 proc SpecSave1D {spec base range type title counts} {
    variable SpecPath

      set f3 [file join $SpecPath $spec.msf]
      set z1 [catch {open $f3 w} m1]
      set z2 [catch {file attributes $f3 -permissions rwxrwxrwx} m2]

      if {$z1 != 0} then {
          puts "Cannot create $f3 - $m1"
      }
      if {$z2 != 0} then {
          puts "error accessing $f3 - $m2"
      }

      set f $m1

      fconfigure $f -translation binary -encoding binary

  set ctime [clock format [clock seconds] -format {%d-%b-%Y %H:%M:%S}]
  set mtime [clock format [clock seconds] -format {%d-%b-%Y %H:%M:%S}]

      puts -nonewline $f [binary format i1 412900921]   ;#   magic number
      puts -nonewline $f [binary format i1 1]           ;#   version
      puts -nonewline $f [binary format a32 $spec]      ;#   spectrum name
      puts -nonewline $f [binary format i1 1]           ;#   dimension
      puts -nonewline $f [binary format a20 $ctime]     ;#   creation time/date
      puts -nonewline $f [binary format a20 $mtime]     ;#   modification time/date
      puts -nonewline $f [binary format i1 $base]      ;#   base0

      for {set i 2} {$i <=8} {incr i} {
         puts -nonewline $f [binary format i1 -1]     ;#   baseI
      }

      puts -nonewline $f [binary format i1 $range]     ;#   range0

      for {set i 2} {$i <=8} {incr i} {
         puts -nonewline $f [binary format i1 -1]     ;#   rangeI
      }

      puts -nonewline $f [binary format i1 -1]        ;#   spectrum title offset

      for {set i 2} {$i <=32} {incr i} {
         puts -nonewline $f [binary format i1 -1]     ;#  information pointers
      }

      for {set i 1} {$i <=8} {incr i} {
         puts -nonewline $f [binary format i1 -1]     ;#  annotation pointers
      }

      for {set i 1} {$i <=8} {incr i} {
         puts -nonewline $f [binary format i1 -1]     ;#  calibration pointers
      }

      for {set i 1} {$i <=8} {incr i} {
         puts -nonewline $f [binary format i1 -1]     ;#  efficiency pointers
      }

                                                     ;#  data array 1 descriptor 
      puts -nonewline $f [binary format i1 0]        ;#  array layout
      puts -nonewline $f [binary format i1 $type]        ;#  array type  (32 bit unsigned)
      puts -nonewline $f [binary format i1 0]        ;#  reserved 
      puts -nonewline $f [binary format i1 0]        ;#  reserved
      puts -nonewline $f [binary format i1 0]        ;#  spectrum counts offset

                                                     ;#  data array 2 descriptor 
      puts -nonewline $f [binary format i1 0]       ;#  array layout
      puts -nonewline $f [binary format i1 $type]       ;#  array type 
      puts -nonewline $f [binary format i1 0]        ;#  reserved 
      puts -nonewline $f [binary format i1 0]        ;#  reserved
      puts -nonewline $f [binary format i1 -1]       ;#  spectrum counts offset

#      puts -nonewline $f [binary format i1 436]      ;#  base of string space
#      puts -nonewline $f [binary format i1 20]       ;#  string free space
#      puts -nonewline $f [binary format i1 76]       ;#  top of string space

      puts -nonewline $f [binary format i1 -1]      ;#  base of string space
      puts -nonewline $f [binary format i1 -1]       ;#  string free space
      puts -nonewline $f [binary format i1 -1]       ;#  top of string space

      puts -nonewline $f [binary format i1 512]      ;#  base of counts space
      puts -nonewline $f [binary format i1 $range]       ;#  string free space
      puts -nonewline $f [binary format i1 $range]       ;#  top of string space
 
#      puts -nonewline $f [binary format a20 $title]     ;# string space - title

      for {set i 436} {$i < 512} {incr i} {
         puts -nonewline $f [binary format c1 0]       ;#  unused
      }

      puts -nonewline $f [binary format i* $counts]      ;# counts

      close $f
}

    proc HistogramSave {} {
       variable SpecBase
       variable SpecPath

        puts "Saving Histograms"

      set f1 $SpecBase
      set z1 [catch {file mkdir $f1} m1]
      set z2 [catch {file attributes $f1 -permissions rwxrwxrwx} m2]

      if {$z1 != 0} then {
          puts "Cannot create $f1 - $m1"
      }
      if {$z2 != 0} then {
          puts "error accessing $f1 - $m2"
      }

      set f1 [file join $f1 [exec uname -n]]
      set z1 [catch {file mkdir $f1} m1]
      set z2 [catch {file attributes $f1 -permissions rwxrwxrwx} m2]

      if {$z1 != 0} then {
          puts "Cannot create $f1 - $m1"
      }
      if {$z2 != 0} then {
          puts "error accessing $f1 - $m2"
      }

      set f2 [file join $f1 [date +%Y%h%d-%H.%M.%S]]
      set z1 [catch {file mkdir $f2} m1]
      set z2 [catch {file attributes $f2 -permissions rwxrwxrwx} m2]

      if {$z1 != 0} then {
          puts "Cannot create $f2 - $m1"
      }
      if {$z2 != 0} then {
          puts "error accessing $f2 - $m2"
      }

       set SpecPath $f2

       set SpecNames [SpectrumNames]
       set NumSpec 0

      foreach item $SpecNames {
         if {[first $item] != 1} {continue}   ;#   not a 1D histogram !!

         set SpecName [second $item]
 
         set info [SpectrumDetails $SpecName]
         set channels [lindex $info 2]

         set title [SpectrumGetTitle $SpecName]
         set counts [SpectrumRead1D $SpecName 0 $channels]

         set z [catch {SpecSave1D $SpecName 0 $channels 5 $title $counts} m]
         if {$z!=0} then {
             puts "Error attempting to save spectrum $SpecName: $m"
         } else {
             incr NumSpec
         }
      }

        puts "$NumSpec Histograms saved"

    }

    proc HistogramZero {} {

       set SpecNames [SpectrumNames]
       set NumSpec 0

       ZeroSpectrum Stat
       incr NumSpec

      foreach item $SpecNames {
         if {[first $item] != 1} {continue}   ;#   not a 1D histogram !!

         set SpecName [second $item]

         ZeroSpectrum $SpecName
         incr NumSpec
      }

       puts "$NumSpec spectra zeroed"
    }

    proc TerminateExec {} {

        unmap_spectra
        DataAcqCommand 22 "Terminating DataAcq program" 0 2

    }

    proc LoadExec {} {

       set z [catch {eval exec nice [file join /MIDAS Data_Acq bin Linux64 CAENExecV5] &} m]
       if {$z != 0} {puts "RunControl LoadExec returned z=$z and $m"}

    }


    foreach module $Supported_Modules {

        set f [file join $env(MIDASBASE) TclHttpd Html CAEN RunControl sys.$module.tcl]
        set z [catch {source $f} m]
        if {$z == 0} {puts "RunControl loaded $f"} else {puts "RunControl returned z=$z and $m"}
    }

    make-UIregisters

    Set_Options

    foreach module $Supported_Modules {Set_[set module]_Options}
