

puts "loading sys.V1724.tcl"

variable registerlist

lappend registerlist [list NetVar.V1724_NumSamples  "-n V1724_NumSamples -t int"]
lappend registerlist [list NetVar.V1724_VMERetries  "-n V1724_VMERetries -t int"]
lappend registerlist [list NetVar.V1724_BERREnable  "-n V1724_BERREnable -t int"]
lappend registerlist [list NetVar.V1724_VMEBufSize  "-n V1724_VMEBufSize -t int"]

variable V1724_Base 160

proc Set_V1724_Options {} {
    variable Options
    variable V1724_Base

puts "Set_V1724_Options"

    load_Options

   if {![info exists Options(V1724_MaxEvtsPerBuf)]}         {set Options(V1724_MaxEvtsPerBuf) 1}
   if {![info exists Options(V1724_MaxBLT)]}                {set Options(V1724_MaxBLT) 1}
   if {![info exists Options(V1724_MemBufSize)]}            {set Options(V1724_MemBufSize) 1024}
   if {![info exists Options(V1724_VMEBufSize)]}            {set Options(V1724_VMEBufSize) 16384}

   if {![info exists Options(V1724_Hist_Trace.Enable)]}    {set Options(V1724_Hist_Trace.Enable) 1}
   if {![info exists Options(V1724_Hist_Trace.channels)]}  {set Options(V1724_Hist_Trace.channels) 512}
   if {![info exists Options(V1724_Hist_Trace.offset)]}    {set Options(V1724_Hist_Trace.offset) 0}
   if {![info exists Options(V1724_Hist_Trace.shift)]}     {set Options(V1724_Hist_Trace.shift) 0}

   if {![info exists Options(V1724_GroupBase)]}             {set Options(V1724_GroupBase) $V1724_Base}
   set V1724_Base  $Options(V1724_GroupBase)

    update_Options
}

proc V1724_Configure {} {
    variable Options

puts "V1724_Configure"

    set VME_BUFFER_SIZE [ReadNetVar NetVar.V1724_VMEBufSize]
    set Options(V1724_VMEBufSize) $VME_BUFFER_SIZE

      update_Options

#    WriteNetVar NetVar.V1724_Trig $Options(V1724_Trig)
#    WriteNetVar NetVar.V1724_DVP  $Options(V1724_DVP)
#    WriteNetVar NetVar.V1724_Diag  $Options(V1724_Diag)

           switch $Options(V1724_MemBufSize) {
              65536 {set BUFF_ORG 2; set BUFF_NUM 4}
              32768 {set BUFF_ORG 3; set BUFF_NUM 8}
              16384 {set BUFF_ORG 4; set BUFF_NUM 16}
              8192  {set BUFF_ORG 5; set BUFF_NUM 32}
              4096  {set BUFF_ORG 6; set BUFF_NUM 64}
              2048  {set BUFF_ORG 7; set BUFF_NUM 128}
              1024  {set BUFF_ORG 8; set BUFF_NUM 256}
              512   {set BUFF_ORG 9; set BUFF_NUM 512}
              256   {set BUFF_ORG 10; set BUFF_NUM 1024}
              default {set BUFF_ORG 10; set BUFF_NUM 1024}
           }

      set z [catch {ReadVMERegisterAttr v1724#1.VMEStatus} m]
      if {$z==0} then {

           set z [catch {WriteVMERegister v1724#1.VMEControl 16} m];  #  force BERR option

           set z [catch {ReadVMERegister v1724#1.CustomSize} SIZE];
           if {$SIZE <= 0} {set SIZE 32}
           WriteNetVar NetVar.V1724_NumSamples $SIZE

           set NUM [expr (($Options(V1724_MemBufSize) - 2) * 2) / $SIZE]
           if {$NUM > $Options(V1724_MaxEvtsPerBuf)} {set NUM $Options(V1724_MaxEvtsPerBuf)}

           set z [catch {ReadVMERegister v1724#1.VMEControl} BERR]  ;#  enable BERR to finish a VME block transfer
           if {$BERR < 0} {set BERR 0}
           WriteNetVar NetVar.V1724_BERREnable [expr ($BERR >> 4) & 1]

puts "V1724 Output buffers $BUFF_NUM  @ $Options(V1724_MemBufSize) 32 bit words"
puts "Waveform [expr $SIZE * 2] samples"
puts "8 channels; $NUM events/buffer"

           set osize [expr (($SIZE + 2) * $NUM) + 2]
puts "V1724 buffer size used $osize words out of max $Options(V1724_MemBufSize)"

           if {$osize > $Options(V1724_MemBufSize)} {
              puts "Number of samples or Events per buffer is too big\n\
                 $osize > $Options(V1724_MemBufSize) - this is not permitted\n\
                 correct options and try again"
              return
           }

           set BLT [expr $VME_BUFFER_SIZE / (($osize * 8) + 4)]
           if {$BLT > $Options(V1724_MaxBLT)} {set BLT $Options(V1724_MaxBLT)}

puts "BLT Event Number = $BLT"
           set isize [expr (($osize * 8) + 4) * $BLT]
puts "Data Acq Input buffer size used $isize words out of max $VME_BUFFER_SIZE"

puts "V1724_Configure $BERR $SIZE $BUFF_ORG $NUM"

           WriteVMERegisters v1724#*.VMEControl $BERR     ;# set all modules to same setting
           WriteVMERegisters v1724#*.CustomSize $SIZE     ;# set all modules to same setting
           WriteVMERegisters v1724#*.BufferOrg $BUFF_ORG
           WriteVMERegisters v1724#*.EvtsPerBuf $NUM
           WriteVMERegisters v1724#*.BLTEventNum $BLT     ;# set all modules to same setting
      }

puts "V1724_Configure finished"

}

proc V1724_DAQ-go {} {

   set maxmodule 0
   for {set i 1} {$i <= 16} {incr i} {
      set z [catch {ReadVMERegisterAttr v1724#$i.VMEStatus} m]
      if {$z!=0} then {break}
      incr maxmodule
   }

   for {set i 1} {$i <= $maxmodule} {incr i} {

      set z [catch {ReadVMERegister v1724#$i.AcqControl} m]
      if {$z == 0} {WriteVMERegister v1724#$i.AcqControl [expr ($m & 0xfb) + 4]}

   }
}

proc V1724_DAQ-halt {} {

   set maxmodule 0
   for {set i 1} {$i <= 16} {incr i} {
      set z [catch {ReadVMERegisterAttr v1724#$i.VMEStatus} m]
      if {$z!=0} then {break}
      incr maxmodule
   }

   for {set i 1} {$i <= $maxmodule} {incr i} {

      set z [catch {ReadVMERegister v1724#$i.AcqControl} m]
      if {$z == 0} {WriteVMERegister v1724#$i.AcqControl [expr ($m & 0xfb)]}

   }
}


proc V1724_DAQ-setup {} {

     V1724_Configure
}

proc V1724_DAQ-flush {} {}

proc V1724_DAQ-reset {} {}

proc Dump_Raw_V1724_Data {} {

   set maxmodule 0
   for {set i 1} {$i <= 16} {incr i} {
      set z [catch {ReadVMERegisterAttr v1724#$i.VMEStatus} m]
      if {$z!=0} then {break}
      incr maxmodule
   }

    set info "Raw data dump for CAEN V1724\n"

    for {set i 1} {$i <= $maxmodule} {incr i} {

       append info "V1724#$i\n"

        DataAcqCommand 19 "Getting Raw Data" "1 $i" 2
        set data [ReadNetVar NetVar.EXEC.Response]

        foreach word $data {
           if {[string range $word 0 1] != "0x"} then {set word 0x$word}
           if {$word == 0xaa5555aa} then {
               append info "$word - end of dump\n"
               break
           }
           if {$word == 0x55aaaa55} then {
               append info "$word - end of event\n"
               continue
           }
           append info "$word "
        }

        append info "\n"
     }

   return $info
}


proc define_V1724_config {} {
    variable Options
    variable V1724_Base

#   set maxmodule [inform vmecontrol inquire v1724count]
#   if {[string length $maxmodule] > 2} {set maxmodule 0}

puts "define_V1724_config"

   set maxmodule 0
   for {set i 1} {$i <= 16} {incr i} {
      set z [catch {ReadVMERegisterAttr v1724#$i.VMEStatus} m]
      if {$z!=0} then {break}
      incr maxmodule
   }

   if {$maxmodule > 0} then {DefineConfig "1724 0 [format "0x%x" $V1724_Base]"}

   for {set i 1} {$i <= $maxmodule} {incr i} {

      set z [catch {ReadVMERegisterAttr v1724#$i.Output_Buffer} m]
      if {$z!=0} then {
          puts "Attempting to locate register v1724#$i.Output_Buffer"
          error $m
      }

      DefineConfig "1724 $i [string range [first $m] 2 end]"
   }
}

proc create_V1724_spectra {} {
    variable V1724_Base
    variable Options 

puts "create_V1724_spectra"

   set maxmodule 0
   for {set i 1} {$i <= 16} {incr i} {
      set z [catch {ReadVMERegisterAttr v1724#$i.VMEStatus} m]
      if {$z!=0} then {break}
      incr maxmodule
   }

   set UseGroup [ReadNetVar NetVar.UseGroup]

   for {set i 1} {$i <= $maxmodule} {incr i} {

     if {$Options(V1724_Hist_Trace.Enable) == 1} {

        for {set j 0} {$j < 8} {incr j} {   

#    histograms for the data trace

           set spectrumname v1724#$i.Ch[set j]
           set z [catch {CreateSpectrum1D $spectrumname $Options(V1724_Hist_Trace.channels)} m]

#    Allow "Spectrum already exists" so that further spectra can be created
           if {$z!=0 && $m==0x30004} then {set z 0}
           if {$z!=0} then {
              puts "Attempting to create spectrum $spectrumname"
              error $m
           } else {
              set group  [expr (($i-1) * 8) + $j + 256  + $V1724_Base] 
              map_spectrum  "$spectrumname $group 0 $Options(V1724_Hist_Trace.offset) $Options(V1724_Hist_Trace.shift)"
           }
        }
     }
   }
}

