
puts "loading sys.V1751.tcl"

variable registerlist

lappend registerlist [list NetVar.V1751_NumSamples  "-n V1751_NumSamples -t int"]
lappend registerlist [list NetVar.V1751_VMERetries  "-n V1751_VMERetries -t int"]
lappend registerlist [list NetVar.V1751_BERREnable  "-n V1751_BERREnable -t int"]
lappend registerlist [list NetVar.V1751_VMEBufSize  "-n V1751_VMEBufSize -t int"]

variable V1751_Base 128

proc Set_V1751_Options {} {
    variable Options
    variable V1751_Base

puts "Set_V1751_Options"

    load_Options

   if {![info exists Options(V1751_MaxEvtsPerBuf)]}         {set Options(V1751_MaxEvtsPerBuf) 1}
   if {![info exists Options(V1751_MaxBLT)]}                {set Options(V1751_MaxBLT) 1}
   if {![info exists Options(V1751_MemBufSize)]}            {set Options(V1751_MemBufSize) 1024}
   if {![info exists Options(V1751_VMEBufSize)]}            {set Options(V1751_VMEBufSize) 16384}

   if {![info exists Options(V1751_Hist_Trace.Enable)]}    {set Options(V1751_Hist_Trace.Enable) 1}
   if {![info exists Options(V1751_Hist_Trace.channels)]}  {set Options(V1751_Hist_Trace.channels) 512}
   if {![info exists Options(V1751_Hist_Trace.offset)]}    {set Options(V1751_Hist_Trace.offset) 0}
   if {![info exists Options(V1751_Hist_Trace.shift)]}     {set Options(V1751_Hist_Trace.shift) 0}

   if {![info exists Options(V1751_GroupBase)]}             {set Options(V1751_GroupBase) $V1751_Base}
   set V1751_Base  $Options(V1751_GroupBase)

    update_Options
}

proc V1751_Configure {} {
    variable Options

puts "V1751_Configure"

    set VME_BUFFER_SIZE [ReadNetVar NetVar.V1751_VMEBufSize]
    set Options(V1751_VMEBufSize) $VME_BUFFER_SIZE

      update_Options

           switch $Options(V1751_MemBufSize) {
              65536 {set BUFF_ORG 2; set BUFF_NUM 4}
              32768 {set BUFF_ORG 3; set BUFF_NUM 8}
              16384 {set BUFF_ORG 4; set BUFF_NUM 16}
              8192  {set BUFF_ORG 5; set BUFF_NUM 32}
              4096  {set BUFF_ORG 6; set BUFF_NUM 64}
              2048  {set BUFF_ORG 7; set BUFF_NUM 128}
              1024  {set BUFF_ORG 8; set BUFF_NUM 256}
              512   {set BUFF_ORG 9; set BUFF_NUM 512}
              256   {set BUFF_ORG 10; set BUFF_NUM 1024}
              default {set BUFF_ORG 10; set BUFF_NUM 1024}
           }

      set z [catch {ReadVMERegisterAttr v1751#1.VMEStatus} m]
      if {$z==0} then {

           set z [catch {WriteVMERegister v1751#1.VMEControl 16} m];  #  force BERR option

           set z [catch {ReadVMERegister v1751#1.CustomSize} SIZE];
           if {$SIZE <= 0} {set SIZE 150}
           WriteNetVar NetVar.V1751_NumSamples [expr $SIZE * 7]  ;# convert to samples

           set WORDS [expr ($SIZE * 7) / 3]   ;# convert to 32 bit words

           set NUM [expr (($Options(V1751_MemBufSize) - 2) * 2) / $WORDS]
           if {$NUM > $Options(V1751_MaxEvtsPerBuf)} {set NUM $Options(V1751_MaxEvtsPerBuf)}

           set z [catch {ReadVMERegister v1751#1.VMEControl} BERR]  ;#  enable BERR to finish a VME block transfer
           if {$BERR < 0} {set BERR 0}
           WriteNetVar NetVar.V1751_BERREnable [expr ($BERR >> 4) & 1]

puts "V1751 Output buffers $BUFF_NUM  @ $Options(V1751_MemBufSize) 32 bit words"
puts "Waveform [expr $SIZE * 7] samples"
puts "8 channels; $NUM events/buffer"

           set osize [expr (($WORDS + 2) * $NUM) + 2]
puts "V1751 buffer size used $osize words out of max $Options(V1751_MemBufSize)"

           if {$osize > $Options(V1751_MemBufSize)} {
              puts "Number of samples or Events per buffer is too big\n\
                 $osize > $Options(V1751_MemBufSize) - this is not permitted\n\
                 correct options and try again"
              return
           }

           set BLT [expr $VME_BUFFER_SIZE / (($osize * 8) + 4)]
           if {$BLT > $Options(V1751_MaxBLT)} {set BLT $Options(V1751_MaxBLT)}

puts "BLT Event Number = $BLT"
           set isize [expr (($osize * 8) + 4) * $BLT]
puts "Data Acq Input buffer size used $isize words out of max $VME_BUFFER_SIZE"

puts "V1751_Configure $BERR $SIZE $BUFF_ORG $NUM"

           WriteVMERegisters v1751#*.VMEControl $BERR     ;# set all modules to same setting
           WriteVMERegisters v1751#*.CustomSize $SIZE     ;# set all modules to same setting
           WriteVMERegisters v1751#*.BufferOrg $BUFF_ORG
           WriteVMERegisters v1751#*.EvtsPerBuf $NUM
           WriteVMERegisters v1751#*.BLTEventNum $BLT     ;# set all modules to same setting
      }

puts "V1751_Configure finished"

}

proc V1751_DAQ-go {} {puts "V1751_DAQ-go"

   set maxmodule 0
   for {set i 1} {$i <= 16} {incr i} {
      set z [catch {ReadVMERegisterAttr v1751#$i.VMEStatus} m]
      if {$z!=0} then {break}
      incr maxmodule
   }

   for {set i 1} {$i <= $maxmodule} {incr i} {

      set z [catch {ReadVMERegister v1751#$i.AcqControl} m]
      if {$z == 0} {WriteVMERegister v1751#$i.AcqControl [expr ($m & 0xfb) + 4]}

   }
}

proc V1751_DAQ-halt {} {puts "V1751_DAQ-halt"

   set maxmodule 0
   for {set i 1} {$i <= 16} {incr i} {
      set z [catch {ReadVMERegisterAttr v1751#$i.VMEStatus} m]
      if {$z!=0} then {break}
      incr maxmodule
   }

   for {set i 1} {$i <= $maxmodule} {incr i} {

      set z [catch {ReadVMERegister v1751#$i.AcqControl} m]
      if {$z == 0} {WriteVMERegister v1751#$i.AcqControl [expr ($m & 0xfb)]}

   }
}

proc V1751_DAQ-setup {} {

   set maxmodule 0
   for {set i 1} {$i <= 16} {incr i} {
      set z [catch {ReadVMERegisterAttr v1751#$i.VMEStatus} m]
      if {$z!=0} then {break}
      incr maxmodule
   }

    if {$maxmodule > 0} then {V1751_Configure}
}

proc V1751_DAQ-flush {} {}

proc V1751_DAQ-reset {} {}

proc define_V1751_config {} {
    variable Options
    variable V1751_Base

#   set maxmodule [inform vmecontrol inquire v1751count]
#   if {[string length $maxmodule] > 2} {set maxmodule 0}

puts "define_V1751_config"

   set maxmodule 0
   for {set i 1} {$i <= 16} {incr i} {
      set z [catch {ReadVMERegisterAttr v1751#$i.VMEStatus} m]
      if {$z!=0} then {break}
      incr maxmodule
   }

   if {$maxmodule > 0} then {DefineConfig "17510 0 [format "0x%x" $V1751_Base]"}

   for {set i 1} {$i <= $maxmodule} {incr i} {

      set z [catch {ReadVMERegisterAttr v1751#$i.Output_Buffer} m]
      if {$z!=0} then {
          puts "Attempting to locate register v1751#$i.Output_Buffer"
          error $m
      }

      DefineConfig "17510 $i [string range [first $m] 2 end]"
   }
}

proc create_V1751_spectra {} {
    variable V1751_Base
    variable Options 

puts "create_V1751_spectra"

   set maxmodule 0
   for {set i 1} {$i <= 16} {incr i} {
      set z [catch {ReadVMERegisterAttr v1751#$i.VMEStatus} m]
      if {$z!=0} then {break}
      incr maxmodule
   }

   for {set i 1} {$i <= $maxmodule} {incr i} {

     if {$Options(V1751_Hist_Trace.Enable) == 1} {

        for {set j 0} {$j < 8} {incr j} {   

#    histograms for the data trace

           set spectrumname v1751#$i.Ch[set j]
           set z [catch {CreateSpectrum1D $spectrumname $Options(V1751_Hist_Trace.channels)} m]

#    Allow "Spectrum already exists" so that further spectra can be created
           if {$z!=0 && $m==0x30004} then {set z 0}
           if {$z!=0} then {
              puts "Attempting to create spectrum $spectrumname"
              error $m
           } else {
              set group  [expr (($i-1) * 8) + $j + 256  + $V1751_Base] 
              map_spectrum  "$spectrumname $group 0 $Options(V1751_Hist_Trace.offset) $Options(V1751_Hist_Trace.shift)"
           }
        }
     }
   }
}


